{==============================================================================}
{ TBinClipboard (C) S. Tkachenko (svt@system.unicor.ac.ru)                     }
{ http://www.crosswinds.net/~svtpro/                                           }
{==============================================================================}
{ TBinClipboard - component for copying/pasting data in custom formats         }
{ to/from Clipboard.                                                           }
{ This component is FREE.                                                      }
{------------------------------------------------------------------------------}
{ Component:                                                                   }
{ Properties:                                                                  }
{ - FormatName - name of your Clipboard format                                 }
{ Methods:                                                                     }
{ - function HasFormat: Boolean; -                                             }
{     Indicates if the Clipboard contains data in format FormatName            }
{ - procedure Copy(Stream: TStream); -                                         }
{     Copies data from stream to Clipboard as format FormatName                }
{ - function Paste(Stream: TStream): Boolean;                                  }
{     Pastes data from Clipboard, if Clipboard contatins data in format        }
{     FormatName. If does not, returns False                                   }
{------------------------------------------------------------------------------}
{ You can also use functions                                                   }
{ - BinClipboardHasFormat                                                      }
{ - BinClipboardCopy                                                           }
{ - BinClipboardPaste                                                          }
{ without component                                                            }
{------------------------------------------------------------------------------}
{ NOTES:                                                                       }
{ - BinClipboardCopy,  BinClipboard.Copy set Stream position to the end;       }
{ - BinClipboardCopy,  BinClipboard.Copy always copy whole Stream data,        }
{   regardless of Stream.Position                                              }
{ - BinClipboardPaste, BinClipboard.Paste set Stream position to 0             }
{ - BinClipboardPaste, BinClipboard.Paste replace current contents in Stream   }
{ - data in clipboard has addition first 4 bytes - length of data (excluding   }
{   these 4 bytes)                                                             }
{ - memory checking tools may report memory leak in BinClipboardCopy;          }
{   ignore - there is no memory leaks in this component.                       }
{==============================================================================}
unit BinClipboard;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Clipbrd;

  function  BinClipboardHasFormat(const FormatName: String): Boolean;
  procedure BinClipboardCopy     (const FormatName: String; Stream: TStream);
  function  BinClipboardPaste    (const FormatName: String; Stream: TStream): Boolean;

type

  TBinClipboard = class(TComponent)
    private
      { Private declarations }
      FFormatName: String;
    protected
      { Protected declarations }
    public
      { Public declarations }
      function HasFormat: Boolean;
      procedure Copy(Stream: TStream);
      function Paste(Stream: TStream): Boolean;
    published
      { Published declarations }
      property FormatName: String read FFormatName write FFormatName;
  end;

  EBinClipboardError = class (Exception);

procedure Register;

implementation
const
  errNoMem = 'Error! No memory?';
  errInvalidData = 'Invalid data in Clipboard!';
{==============================================================================}
function  BinClipboardHasFormat(const FormatName: String): Boolean;
begin
  Result := Clipboard.HasFormat(RegisterClipboardFormat(PChar(FormatName)));
end;
{------------------------------------------------------------------------------}
procedure BinClipboardCopy(const FormatName: String; Stream: TStream);
var mem: Cardinal;
    ptr: PChar;
    Size: Integer;
begin
  Size := Stream.Size;
  mem := GlobalAlloc(GMEM_MOVEABLE or GMEM_DDESHARE, SizeOf(Size)+Size);
  if mem=0 then
    raise EBinClipboardError.Create(errNoMem);
  ptr := PChar(GlobalLock(mem));
  Move(Size,           ptr^,                SizeOf(Size));
  Stream.Position := 0;
  Stream.ReadBuffer((ptr+SizeOf(Size))^, Size);
  GlobalUnlock(mem);
  Clipboard.SetAsHandle(RegisterClipboardFormat(PChar(FormatName)),mem);
end;
{------------------------------------------------------------------------------}
function  BinClipboardPaste(const FormatName: String; Stream: TStream): Boolean;
var cf: Cardinal;
    mem: Cardinal;
    ptr: PChar;
    Size, MemSize: Integer;
begin
  Result := False;
  cf := RegisterClipboardFormat(PChar(FormatName));
  if not Clipboard.HasFormat(cf) then exit;
  Clipboard.Open;
  try
    mem := Clipboard.GetAsHandle(cf);
    if mem=0 then
      raise EBinClipboardError.Create(errNoMem);
    MemSize := GlobalSize(mem);
    if MemSize<SizeOf(Size) then
      raise EBinClipboardError.Create(errInvalidData);
    ptr := PChar(GlobalLock(mem));
    try
      Move(ptr^,Size, SizeOf(Size));
      if MemSize<SizeOf(Size)+Size then
        raise EBinClipboardError.Create(errInvalidData);
      Stream.Size := Size;
      Stream.Position := 0;
      Stream.WriteBuffer((ptr+SizeOf(Size))^, Size);
      Stream.Position := 0;      
    finally
      GlobalUnlock(mem);
    end;
  finally
    Clipboard.Close;
  end;
  Result := True;
end;
{================================ TBinClipboard ===============================}
function TBinClipboard.HasFormat: Boolean;
begin
  Result := BinClipboardHasFormat(FormatName);
end;
{------------------------------------------------------------------------------}
procedure TBinClipboard.Copy(Stream: TStream);
begin
  BinClipboardCopy(FormatName, Stream);
end;
{------------------------------------------------------------------------------}
function TBinClipboard.Paste(Stream: TStream): Boolean;
begin
  Result := BinClipboardPaste(FormatName, Stream);
end;
{==============================================================================}
procedure Register;
begin
  RegisterComponents('SVTPro', [TBinClipboard]);
end;
{==============================================================================}
end.
